function ssw_getcme_list, time0, time1, _extra=_extra, refresh=refresh, $ 
   cdaw=cdaw, debug=debug, search_array=search_array, count=count
;
;+
;   Name: ssw_getcme_list
;
;   Purpose: return LASCO CME list info for users time range; optionl search
;
;   Input Parameters:
;      time0 - optional  time or start time of range (mission list if none)      
;      time1 - stop time if range
;
;   Output:
;      function retruns list as structure, including ssw compliant times
;
;   Keyword Parameters:
;      cdaw - if set, use list at cdaw.gsfc.nasa.gov (default..)
;      count - number of elements (cme matches/structures) returned
;      search_array - optional search strings (see struct_where.pro)   
;                     (valid tags include {cpa, width, lspeed, accel,remarks})
; 
;   Calling Examples:
;      cmes=ssw_getcme_list('15-mar-2001','15-jul-2001') ; all within range
;      halos=ssw_get_cme_list(search=['width=360'])      ; mission Halos
;      partial=ssw_getcme_list('1-jan-2005','1-feb-2005', $
;         search=['width=180~360','lspeed>1000',''accel>10'])
;
;   Context Example: - show times of all Halo events in list via utplot
;      IDL> halos=ssw_getcme_list(search=['width=360']) ; mission search
;      IDL> utplot,halos, halos.lspeed,psym=2
;         
;   Method:
;      1st call - socket access list -> convert to "ssw" structure 
;                 subsequent, use common (/REFRESH to force recopy->structs)
;
;   Restrictions:
;      more of a comment - 1st call in session will take a minute or two
;      subsequent calls Much faster
;      Only CDAW list by Gopalswamy et al available as of today
;      (plan: extend to CACTUS list option)
;
;
;   History:
;      6-Sep-2005 - S.L.Freeland  
;     14-Sep-2005 - S.L.Freeland - change 2nd order field names; def vals -1 -> -99999


common ssw_getcme_list_blk,cmestr_mission

debug=keyword_set(debug)
refresh=keyword_set(refresh) or n_elements(cmestr_mission) eq 0

cdaw_url='http://cdaw.gsfc.nasa.gov/CME_list/UNIVERSAL/text_ver/univ_all.txt'
if refresh then begin 
   case 1 of 
      keyword_set(cdaw): top=cdaw_url
      else: begin 
         box_message,'CDAW/lasco list only for now - using that'
         top=cdaw_url
      endcase
   endcase
   box_message,'1st access and refresh; url->structure... please be patient
   sock_list,top,list
   if n_elements(list) lt 25 then begin 
      box_message,'Problem listing '+top
      return,''
   endif
  
   topss=(where(is_number(strmid(list,0,1))))(0) ; start of valid data
   lcols=str2cols(list(topss:*),/trim)
   strtab2vect,lcols,date,time,cpa,width,lspeed,inits,finals,second_order,accel,mpa,remarks

   cnvts=str2arr('cpa,width,lspeed,inits,finals,second_order,accel')
   for i=0,n_elements(cnvts)-1 do begin
 
      estat=execute(cnvts(i)+"(where(1-is_number("+cnvts(i)+")))='-99999'")
      estat=execute(cnvts(i)+"=float("+cnvts(i)+")")
   endfor
   cmestr={Anytim_dobs:0.d,mjd:0l,time:0l,cpa:0,width:0,lspeed:0,$
       qspeed_init:0,qspeed_final:0,qspeed_2or:0,accel:0.0,mpa:0,remarks:''}
   cmestr_mission=replicate(cmestr,n_elements(date))
   dobs=anytim(date + ' ' + time,/utc_int)
   cmestr_mission.mjd=dobs.mjd
   cmestr_mission.time=dobs.time
   cmestr_mission.anytim_dobs=anytim(dobs)
   cmestr_mission.cpa=fix(cpa)
   cmestr_mission.width=fix(width)
   cmestr_mission.lspeed=fix(lspeed)
   cmestr_mission.qspeed_init=fix(inits)
   cmestr_mission.qspeed_final=fix(finals)
   cmestr_mission.qspeed_2or=fix(second_order)
   cmestr_mission.accel=accel
   cmestr_mission.mpa=fix(mpa)
   cmestr_mission.remarks=remarks
   
endif 
sscnt=-1
case n_params() of
   0: retval=cmestr_mission
   1: begin    
        dt=abs(anytim(time0) - cmestr_mission.anytim_dobs)
        sss=(where(dt eq min(dt),sscnt))(0)
   endcase
   2: begin 
       sss=where(anytim(time0) le cmestr_mission.anytim_dobs and $ 
                  anytim(time1) ge cmestr_mission.anytim_dobs,sscnt)
   endcase
endcase

if sscnt gt 0 then retval=cmestr_mission(sss)

; now perform optional searches...

if data_chk(search_array,/string) then begin 
   sss=struct_where(retval,test=search_array, count)
   if count eq 0 then begin 
      box_message,['No matches for your search criteria:',search_array]
stop
      retval=''
   endif else retval=retval(sss)
endif

count=n_elements(retval) * (data_chk(retval,/struct))    

if debug then stop,'before return'

return,retval
end
